extends Sprite

enum ColorOrder {
	RED = 0xFF000000,
	GREEN = 0x00FF0000,
	BLUE = 0x0000FF00
}

var update_shader = true				# Generate shader code on first frame
var current_color = ColorOrder.RED		# Initial color
var current_color_intensity = 0.75		# Initial color intensity

func _process(delta):

	if (Input.is_action_pressed("color_intensity_up")):

		current_color_intensity += 0.5 * delta
		update_shader = true

	if (Input.is_action_pressed("color_intensity_down")):

		current_color_intensity -= 0.5 * delta
		update_shader = true

	if (update_shader):
		material.shader.code = generate_shader_code(generate_color(current_color, current_color_intensity))
		$ShaderCodeLabel.text = material.shader.code

func _input(event):

	if (event.is_action_pressed("change_color")):

		current_color = get_next_color(current_color)
		update_shader = true

func generate_shader_code(color):

	return "shader_type canvas_item;\nvoid fragment() { COLOR = texture(TEXTURE, UV) + vec4(%f, %f, %f, %f); }" \
			% [color.r, color.g, color.b, color.a]

func get_next_color(color_enum):

	if (color_enum == ColorOrder.BLUE):
		return ColorOrder.RED

	return color_enum >> 8

func generate_color(color_enum, intensity):

	return Color(0xFFFFFFFF & color_enum) * intensity